/*
 * This file is part of unity-2d
 *
 * Copyright 2010-2011 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 1.0
import Unity2d 1.0
import "../common"
import "../common/units.js" as Units
import "../common/utils.js" as Utils

Renderer {
    needHeader: true
    currentItem: flow.currentItem
    /* FIXME: should be defined as a property of Renderer */
    property int contentHeight: Units.tvPx(540)

    ListView {
        id: flow

        property real margin: Units.tvPx(90)
        property real itemWidth: Units.tvPx(210)

        property int duration: 120

        anchors.fill: parent
        anchors.topMargin: Units.tvPx(60)
        anchors.bottomMargin: Units.tvPx(60)
        /* Make sure the first and last items are offsetted by margin */
        anchors.leftMargin: margin
        anchors.rightMargin: margin
        cacheBuffer: margin*2
        focus: true
        orientation: ListView.Horizontal
        spacing: Units.tvPx(56)
        highlightMoveSpeed: -1
        highlightMoveDuration: 300
        highlightRangeMode: ListView.ApplyRange
        preferredHighlightBegin: margin + itemWidth
        preferredHighlightEnd: width - margin - itemWidth
        snapMode: ListView.SnapToItem
        model: category_model

        delegate: AbstractButton {
            id: button

            property string uri: column_0
            property string iconHint: column_1
            property string categoryIndex: column_2
            property string mimetype: column_3
            property string name: column_4
            property string comment: column_5
            property string dndUri: column_6

            width: flow.itemWidth
            height: flow.height

            property variant segments: [-flow.itemWidth, 0.0, flow.width-flow.itemWidth, flow.width-flow.itemWidth+flow.itemWidth]
            property real absoluteX: button.x-flow.contentX

            property real angle
            angle: Utils.segmentsLinearInterpolation(segments, [63.0, 0.0, 0.0, -63.0], absoluteX)
            property real origin
            origin: Utils.segmentsLinearInterpolation(segments, [width, width, 0.0, 0.0], absoluteX)
            //Behavior on angle {SmoothedAnimation {duration: flow.duration; velocity: -1}}
            transform: Rotation { origin.x: button.origin; origin.y: height/2; axis { x: 0; y: 1; z: 0 } angle: button.angle}

            scale: Utils.segmentsLinearInterpolation(segments, [0.8, 1.0, 1.0, 0.8], absoluteX)
            //Behavior on scale {SmoothedAnimation {duration: flow.duration; velocity: -1}}

            onClicked: dash.activateUriWithLens(lens, categoryId, uri, mimetype)

            Item {
                id: iconImage

                anchors.top: parent.top
                height: Units.tvPx(300)
                anchors.left: parent.left
                anchors.right: parent.right

                Item {
                    width: image.paintedWidth
                    height: image.paintedHeight
                    anchors.centerIn: image

                    Image {
                        anchors.verticalCenter: parent.bottom
                        anchors.horizontalCenter: parent.horizontalCenter
                        width: Units.tvPx(330)
                        height: Units.tvPx(80)
                        source: "artwork/coverflow_shadow.png"
                        fillMode: Image.PreserveAspectFit
                    }

                    Image {
                        source: "artwork/selection_glow.png"
                        anchors.fill: parent
                        anchors.margins: Units.tvPx(-37)
                        opacity: button.activeFocus ? 1.0 : 0.0
                        Behavior on opacity {NumberAnimation {duration: 200; easing.type: Easing.OutQuad}}
                    }

                    Rectangle {
                        id: outline
                        anchors.fill: parent
                        anchors.margins: Units.tvPx(-2)
                        color: "#00000000"
                        smooth: true
                        border.width: Units.tvPx(4)
                        border.color: "black"
                        radius: Units.tvPx(3)
                    }
                }


                Image {
                    id: image

                    anchors.fill: parent
                    fillMode: Image.PreserveAspectFit

                    source: "image://icons/" + iconHint
                    sourceSize.width: width
                    sourceSize.height: height
                    smooth: true
                    asynchronous: true
                }
            }

            TextMultiLine {
                id: label

                anchors.top: iconImage.bottom
                anchors.bottom: parent.bottom
                anchors.right: parent.right
                anchors.left: parent.left
                anchors.topMargin: Units.tvPx(35)

                text: name
                opacity: Utils.segmentsLinearInterpolation(segments, [0.0, 1.0, 1.0, 0.0], absoluteX)
                //Behavior on opacity {SmoothedAnimation {duration: flow.duration; velocity: -1}}
                color: "#ffffff"
                horizontalAlignment: Text.AlignHCenter
                fontSize: "small"
                style: Text.Raised
                styleColor: "#1e1e1e"
            }
        }
    }
}
